@@ -1937,6 +1937,9 @@ public class HTTPConduit
                 && !"PUT".equals(connection.getRequestMethod())) {
                 return;
             }
+            if (outMessage.get("org.apache.cxf.post.empty") != null) {
+                return;
+            }
             
             // If we need to cache for retransmission, store data in a
             // CacheAndWriteOutputStream. Otherwise write directly to the output stream.
@@ -1977,13 +1980,40 @@ public class HTTPConduit
             }
             try {
                 handleResponse();
+            } catch (IOException e) {
+                String url = connection.getURL().toString();
+                String origMessage = e.getMessage();
+                if (origMessage != null && origMessage.contains(url)) {
+                    throw e;
+                }
+                throw mapException(e.getClass().getSimpleName() 
+                                   + " invoking " + connection.getURL() + ": "
+                                   + e.getMessage(), e,
+                                   IOException.class);
+            } catch (RuntimeException e) {
+                throw mapException(e.getClass().getSimpleName() 
+                                   + " invoking " + connection.getURL() + ": "
+                                   + e.getMessage(), e,
+                                   RuntimeException.class);
             } finally {
                 if (cachingForRetransmission && cachedStream != null) {
                     cachedStream.close();
                 }
             }
         }
-        
+        private <T extends Exception> T mapException(String msg, 
+                                                     T ex, Class<T> cls) {
+            T ex2 = ex;
+            try {
+                ex2 = cls.cast(ex.getClass().getConstructor(String.class).newInstance(msg));
+                ex2.initCause(ex);
+            } catch (Throwable e) {
+                ex2 = ex;
+            }
+            
+            
+            return ex2;
+        }
         
         /**
          * This procedure handles all retransmits, if any.
@@ -2087,7 +2117,7 @@ public class HTTPConduit
                 LOG.fine("Content length: " + connection.getContentLength());
                 Map<String, List<String>> headerFields = connection.getHeaderFields();
                 if (null != headerFields) {
-                    StringBuffer buf = new StringBuffer();
+                    StringBuilder buf = new StringBuilder();
                     buf.append("Header fields: ");
                     buf.append(System.getProperty("line.separator"));
                     for (String h : headerFields.keySet()) {
@@ -2103,7 +2133,8 @@ public class HTTPConduit
         
             
             if (responseCode == HttpURLConnection.HTTP_NOT_FOUND) {
-                throw new IOException(connection.getResponseMessage());
+                throw new IOException("HTTP response '" + responseCode + ": " 
+                        + connection.getResponseMessage() + "'");
             }
 
             
