@@ -33,10 +33,12 @@ import java.util.Map;
 
 import javax.annotation.PostConstruct;
 import javax.annotation.PreDestroy;
+import javax.servlet.http.HttpServletResponse;
 import javax.ws.rs.Consumes;
 import javax.ws.rs.DELETE;
 import javax.ws.rs.DefaultValue;
 import javax.ws.rs.GET;
+import javax.ws.rs.HEAD;
 import javax.ws.rs.HeaderParam;
 import javax.ws.rs.MatrixParam;
 import javax.ws.rs.POST;
@@ -94,6 +96,12 @@ public class BookStore {
         System.out.println("PreDestroy called");
     }
     
+    @POST
+    @Path("emptypost")
+    public void emptypost() {
+        System.out.println("empty post");
+    }
+    
     @GET
     @Path("webappexception")
     public Book throwException() {
@@ -115,6 +123,14 @@ public class BookStore {
         throw new BookNotFoundFault("Book Exception");
     }
     
+    @GET
+    @Path("propogateexception3")
+    public Book propogateException3() throws BookNotFoundFault {
+        PhaseInterceptorChain.getCurrentMessage().getExchange()
+            .put("org.apache.cxf.systest.for-out-fault-interceptor", Boolean.TRUE);
+        throw new BookNotFoundFault("Book Exception");
+    }
+    
     @GET
     @Path("books/check/{id}")
     @Produces("text/plain")
@@ -233,6 +249,18 @@ public class BookStore {
         return doGetBook(id);
     }
     
+    @GET
+    @Path("/getheadbook/")
+    public Book getBookGetHead() throws Exception {
+        return doGetBook("123");
+    }
+    
+    @HEAD
+    @Path("/getheadbook/")
+    public Response getBookGetHead2() throws Exception {
+        return Response.ok().header("HEAD_HEADER", "HEAD_HEADER_VALUE").build();
+    }
+    
     
     @GET
     @Path("/bookquery")
@@ -446,6 +474,15 @@ public class BookStore {
         return Response.ok(book).build();
     }
     
+    @POST
+    @Path("/books/customstatus")
+    @Produces("text/xml")
+    @Consumes("text/xml")
+    public Response addBookCustomFailure(Book book, @Context HttpServletResponse response) {
+        response.setStatus(333);
+        return null;
+    }
+    
     @POST
     @Path("/booksinfo")
     @Produces("text/xml")
