@@ -75,6 +75,20 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                       data, "application/xml", 500);
     }
     
+    @Test
+    public void testPropogateException3() throws Exception {
+        String data = "<nobook/>";
+        getAndCompare("http://localhost:9080/bookstore/propogateexception3",
+                      data, "application/xml", 500);
+    }
+    
+    @Test
+    public void testPropogateException4() throws Exception {
+        String data = "<nobook/>";
+        getAndCompare("http://localhost:9080/bookstore/propogateexception4",
+                      data, "application/xml", 500);
+    }
+    
     @Test
     public void testWebApplicationException() throws Exception {
         getAndCompare("http://localhost:9080/bookstore/webappexception",
@@ -192,6 +206,14 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         assertEquals(0, ((InputStream)response.getEntity()).available());
     }
     
+    @Test
+    public void testEmptyPost() throws Exception {
+        WebClient wc = 
+            WebClient.create("http://localhost:9080/bookstore/emptypost");
+        Response response = wc.post(null);
+        assertEquals(204, response.getStatus());
+    }
+    
     @Test
     public void testGetBookByEncodedQuery() throws Exception {
         getAndCompareAsStrings("http://localhost:9080/bookstore/bookquery?"
@@ -288,8 +310,8 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     
     @Test
     public void testNoMessageWriterFound() throws Exception {
-        String msg1 = ".No message body writer found for response class : GregorianCalendar.";
-        String msg2 = ".No message body writer found for response class : Calendar.";
+        String msg1 = "No message body writer has been found for response class GregorianCalendar.";
+        String msg2 = "No message body writer has been found for response class Calendar.";
         
         getAndCompareStrings("http://localhost:9080/bookstore/timetable", 
                              new String[]{msg1, msg2}, "*/*", 500);
@@ -372,28 +394,44 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     }
     
     @Test
-    public void testGetBook123WebClient() throws Exception {
+    public void testGetHeadBook123WebClient() throws Exception {
+        String address = "http://localhost:9080/bookstore/getheadbook/";
+        WebClient client = WebClient.create(address);
+        Response r = client.head();
+        assertEquals("HEAD_HEADER_VALUE", r.getMetadata().getFirst("HEAD_HEADER"));
+    }
+    
+    @Test
+    public void testGetHeadBook123WebClient2() throws Exception {
+        String address = "http://localhost:9080/bookstore/getheadbook/";
+        WebClient client = WebClient.create(address);
+        Book b = client.get(Book.class);
+        assertEquals(b.getId(), 123L);
+    }
+    
+    @Test
+    public void testGetBook123WithProxy() throws Exception {
         BookStore bs = JAXRSClientFactory.create("http://localhost:9080", BookStore.class);
         Book b = bs.getBook("123");
         assertEquals(b.getId(), 123);
     }
     
     @Test
-    public void testDeleteWithWebClient() throws Exception {
+    public void testDeleteWithProxy() throws Exception {
         BookStore bs = JAXRSClientFactory.create("http://localhost:9080", BookStore.class);
         Response r = bs.deleteBook("123");
         assertEquals(200, r.getStatus());
     }
     
     @Test
-    public void testCreatePut() throws Exception {
+    public void testCreatePutWithProxy() throws Exception {
         BookStore bs = JAXRSClientFactory.create("http://localhost:9080", BookStore.class);
         Response r = bs.createBook(777L);
         assertEquals(200, r.getStatus());
     }
     
     @Test
-    public void testUpdateWithWebClient() throws Exception {
+    public void testUpdateWithProxy() throws Exception {
         BookStore bs = JAXRSClientFactory.create("http://localhost:9080", BookStore.class);
         Book book = new Book();
         book.setId(888);
@@ -413,6 +451,7 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                                "application/xml", 200);
     }
     
+    
     @Test
     public void testGetBook123() throws Exception {
         getAndCompareAsStrings("http://localhost:9080/bookstore/books/123",
@@ -538,7 +577,7 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         getAndCompareAsStrings(
             "http://localhost:9080/bookstore/booksubresourceobject/123/chaptersobject/sub/1",
             "resources/expected_get_chapter1.txt", "application/xml",
-            "application/xml;charset=iso-8859-1", 200);
+            "application/xml;charset=ISO-8859-1", 200);
     }
     
     @Test
@@ -546,7 +585,7 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         
         getAndCompareAsStrings("http://localhost:9080/bookstore/booksubresource/123/chapters/1",
                                "resources/expected_get_chapter1.txt",
-                               "application/xml", "application/xml;charset=iso-8859-1", 200);
+                               "application/xml", "application/xml;charset=ISO-8859-1", 200);
     }
     
     @Test
@@ -565,7 +604,7 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                                "application/xml", 200);
         getAndCompareAsStrings("http://localhost:9080/bookstore/booksubresource/123/chapters/sub/1/recurse2",
                                "resources/expected_get_chapter1.txt",
-                               "application/xml", "application/xml;charset=iso-8859-1", 200);
+                               "application/xml", "application/xml;charset=ISO-8859-1", 200);
     }
     
     @Test
@@ -574,7 +613,7 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         getAndCompareAsStrings(
             "http://localhost:9080/bookstore/booksubresource/123/chapters/sub/1/recurse2/ids",
             "resources/expected_get_chapter1.txt",
-            "application/xml", "application/xml;charset=iso-8859-1", 200);
+            "application/xml", "application/xml;charset=ISO-8859-1", 200);
     }
     
     @Test
@@ -584,6 +623,21 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                                "text/plain", 200);
     }
     
+    @Test
+    public void testAddBookNoBody() throws Exception {
+        PostMethod post = new PostMethod("http://localhost:9080/bookstore/books");
+        post.setRequestHeader("Content-Type", "application/xml");
+        HttpClient httpclient = new HttpClient();
+        
+        try {
+            int result = httpclient.executeMethod(post);
+            assertEquals(400, result);
+        } finally {
+            // Release current connection to the connection pool once you are done
+            post.releaseConnection();
+        }
+    }
+    
     @Test
     public void testAddBook() throws Exception {
         doAddBook("http://localhost:9080/bookstore/books");               
@@ -615,6 +669,27 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         }
     }
     
+    
+    @Test
+    public void testAddBookCustomFailureStatus() throws Exception {
+        String endpointAddress = "http://localhost:9080/bookstore/books/customstatus";
+
+        File input = new File(getClass().getResource("resources/update_book.txt").toURI());
+        PostMethod put = new PostMethod(endpointAddress);
+        RequestEntity entity = new FileRequestEntity(input, "text/xml; charset=ISO-8859-1");
+        put.setRequestEntity(entity);
+        HttpClient httpclient = new HttpClient();
+
+        try {
+            int result = httpclient.executeMethod(put);
+            assertEquals(333, result);
+        } finally {
+            // Release current connection to the connection pool once you are
+            // done
+            put.releaseConnection();
+        }
+    }
+    
     @Test
     public void testUpdateBook() throws Exception {
         String endpointAddress = "http://localhost:9080/bookstore/books";
