@@ -88,6 +88,30 @@ public class JAXRSMultipartTest extends AbstractBusClientServerTestBase {
         doAddBook(address, "attachmentData", 200);               
     }
     
+    @Test
+    public void testBookJSONForm() throws Exception {
+        String address = "http://localhost:9085/bookstore/books/jsonform";
+        doAddFormBook(address, "attachmentFormJson", 200);               
+    }
+    
+    @Test
+    public void testBookJaxbForm() throws Exception {
+        String address = "http://localhost:9085/bookstore/books/jaxbform";
+        doAddFormBook(address, "attachmentFormJaxb", 200);               
+    }
+    
+    @Test
+    public void testBookJSONJAXBForm() throws Exception {
+        String address = "http://localhost:9085/bookstore/books/jsonjaxbform";
+        doAddFormBook(address, "attachmentFormJsonJaxb", 200);               
+    }
+    
+    @Test
+    public void testBookJSONFormFiles() throws Exception {
+        String address = "http://localhost:9085/bookstore/books/filesform";
+        doAddFormBook(address, "attachmentFormJsonFiles", 200);               
+    }
+    
     @Test
     public void testBookAsMessageContextAttachment() throws Exception {
         String address = "http://localhost:9085/bookstore/books/attachment";
@@ -355,6 +379,22 @@ public class JAXRSMultipartTest extends AbstractBusClientServerTestBase {
         assertEquals("java.jpg", cd2.getParameter("filename"));
     }
     
+    @Test
+    public void testMultipartRequestNoBody() throws Exception { 
+        PostMethod post = new PostMethod("http://localhost:9085/bookstore/books/image");
+        String ct = "multipart/mixed";
+        post.setRequestHeader("Content-Type", ct);
+        HttpClient httpclient = new HttpClient();
+        
+        try {
+            int result = httpclient.executeMethod(post);
+            assertEquals(400, result);
+        } finally {
+            // Release current connection to the connection pool once you are done
+            post.releaseConnection();
+        }
+    }
+    
     private void doAddBook(String address, String resourceName, int status) throws Exception {
         doAddBook("multipart/related", address, resourceName, status);
     }
@@ -384,6 +424,30 @@ public class JAXRSMultipartTest extends AbstractBusClientServerTestBase {
         }
     }
     
+    private void doAddFormBook(String address, String resourceName, int status) throws Exception {
+        PostMethod post = new PostMethod(address);
+        
+        String ct = "multipart/form-data; boundary=bqJky99mlBWa-ZuqjC53mG6EzbmlxB";
+        post.setRequestHeader("Content-Type", ct);
+        InputStream is = 
+            getClass().getResourceAsStream("/org/apache/cxf/systest/jaxrs/resources/" + resourceName);
+        RequestEntity entity = new InputStreamRequestEntity(is);
+        post.setRequestEntity(entity);
+        HttpClient httpclient = new HttpClient();
+        
+        try {
+            int result = httpclient.executeMethod(post);
+            assertEquals(status, result);
+            if (status == 200) {
+                InputStream expected = getClass().getResourceAsStream("resources/expected_add_book.txt");
+                assertEquals(getStringFromInputStream(expected), post.getResponseBodyAsString());
+            }
+        } finally {
+            // Release current connection to the connection pool once you are done
+            post.releaseConnection();
+        }
+    }
+    
     private String getStringFromInputStream(InputStream in) throws Exception {        
         CachedOutputStream bos = new CachedOutputStream();
         IOUtils.copy(in, bos);
