@@ -24,6 +24,7 @@ import java.io.File;
 import java.io.FileInputStream;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
+import java.util.Arrays;
 import java.util.List;
 
 import javax.jws.WebService;
@@ -177,7 +178,7 @@ public class CodeGenBugTest extends AbstractCodeGenTest {
 
         File cxf = new File(org, "cxf");
         File[] files = cxf.listFiles();
-        assertEquals(23, files.length);
+        assertEquals(25, files.length);
 
         Class clz = classLoader.loadClass("org.cxf.Greeter");
         assertTrue("Generate " + clz.getName() + "error", clz.isInterface());
@@ -576,7 +577,7 @@ public class CodeGenBugTest extends AbstractCodeGenTest {
         processor.execute();
 
         File file = new File(output, "org/apache/cxf/w2j/hello_world_soap_http");
-        assertEquals(4, file.list().length);
+        assertEquals(Arrays.asList(file.list()).toString(), 4, file.list().length);
         file = new File(output,
                         "org/apache/cxf/w2j/hello_world_soap_http/DocLitBare_DocLitBarePort_Client.java");
         assertTrue("DocLitBare_DocLitBarePort_Client is not found", file.exists());
@@ -935,7 +936,7 @@ public class CodeGenBugTest extends AbstractCodeGenTest {
         processor.setContext(env);
         processor.execute();
         File file = new File(output, "org/mypkg");
-        assertEquals(23, file.listFiles().length);
+        assertEquals(25, file.listFiles().length);
         Class<?> clz = classLoader.loadClass("org.mypkg.MyService");
         assertNotNull("Customized service class is not found", clz);
         clz = classLoader.loadClass("org.mypkg.MyGreeter");
@@ -961,7 +962,7 @@ public class CodeGenBugTest extends AbstractCodeGenTest {
         assertTrue(results1.contains("     * this is method javadoc"));
         
         List<String> results2 = FileUtils.readLines(new File(output.getCanonicalPath(), 
-            "org/mypkg/SOAPService.java")); 
+            "org/mypkg/SOAPServiceTest1.java")); 
         
         boolean match1 = false;
         boolean match2 = false;
@@ -969,12 +970,12 @@ public class CodeGenBugTest extends AbstractCodeGenTest {
             if (str.contains("package javadoc")) {
                 match1 = true;
             }
-            if (str.contains("class javadoc")) {
+            if (str.contains("service class javadoc")) {
                 match2 = true;
             }
         }
-        assertTrue(match1);
-        assertTrue(match2);
+        assertTrue(results1.toString(), match1);
+        assertTrue(results2.toString(), match2);
         
     }
     
@@ -1071,5 +1072,37 @@ public class CodeGenBugTest extends AbstractCodeGenTest {
         assertTrue(file.exists());
         String str = FileUtils.getStringFromFile(file);
         assertTrue(str.contains("http://child/xsd"));
-    }   
+    }
+    
+    @Test
+    public void testMultiXjcArgs() throws Exception {
+        String[] args = new String[] {"-d", output.getCanonicalPath(),
+                                      "-xjc-Xlocator",
+                                      "-xjc-Xsync-methods",
+                                      getLocation("/wsdl2java_wsdl/hello_world.wsdl")};
+        WSDLToJava.main(args);
+        File file = new File(output, "org/apache/cxf/w2j/hello_world_soap_http/types/SayHi.java");
+        
+        assertTrue(file.exists());
+        String str = FileUtils.getStringFromFile(file);
+        assertTrue(str.contains("@XmlLocation"));
+        assertTrue(str.contains("synchronized"));
+    }
+    
+    @Test
+    public void testCXF1939() throws Exception {
+        String[] args = new String[] {"-d", output.getCanonicalPath(),
+            "-impl", "-server", "-client", "-autoNameResolution",
+            getLocation("/wsdl2java_wsdl/cxf1939/hello_world.wsdl")};
+        WSDLToJava.main(args);
+                
+        assertNotNull(output);
+        assertTrue(new File(output, "org/apache/cxf/w2j/hello_world_soap_http/GreeterImpl.java").exists());
+        assertTrue(new File(output, "org/apache/cxf/w2j/hello_world_soap_http/GreeterImpl1.java").exists());
+        assertTrue(new File(output, 
+                            "org/apache/cxf/w2j/hello_world_soap_http/TestServiceName.java").exists());
+        assertTrue(new File(output, 
+                            "org/apache/cxf/w2j/hello_world_soap_http/TestServiceName1.java").exists());
+    }
+
 }
